/**
 * @fileOverview 输出一个最基本的、仅包含核心功能的表格类。
 */
import TinyEmitter from 'tinyemitter';
import './style.css';
import { GridPlace, FixedGrids } from '../plugins/fixed-table';
import { SortBlock } from '../plugins/sort';
export declare type DataGridConstructor = new (...args: any[]) => BaseGrid;
export interface ColumnObj {
    key: string;
}
export declare type Column = string | ColumnObj;
export interface Row {
    [prop: string]: any;
}
export interface DataGridOptions {
    th?: (column: ColumnObj, index: number) => string | Node;
    td?: (column: ColumnObj, row: Row, columnIndex: number, rowIndex: number) => string | Node;
    parent?: BaseGrid;
    sortBlock?: SortBlock;
    [other: string]: any;
}
export interface InnerDataGridOptions extends DataGridOptions {
    [prop: string]: any;
}
export interface TableData {
    columns: Column[];
    rows: Row[];
}
export default class BaseGrid extends TinyEmitter {
    readonly el: HTMLDivElement;
    readonly ui: {
        [prop: string]: HTMLElement;
    };
    protected options: InnerDataGridOptions;
    curData: TableData;
    /** 如果当前实例用了 fixedTable 插件，则会有这个属性 */
    protected children?: BaseGrid[];
    protected fixedTables?: FixedGrids;
    /** 如果当前实例是 fixedTable 创建的内部表格则会有这个属性 */
    readonly parent?: BaseGrid;
    /** 这个固定表格当前固定的列的个数 */
    fixedColumns?: number;
    /** 这个固定表格的位置 */
    fixedPlace?: GridPlace;
    constructor(options?: DataGridOptions);
    /** 根据数据生成表格内容。 */
    setData(data: TableData): void;
    /** 显示一段消息。默认会显示“暂无数据” */
    showModal(html?: string): void;
    /** 隐藏消息。 */
    hideModal(): void;
    /**
     * 销毁对象。
     * @param remove 如果为 true，则从 DOM 中删除元素。
     */
    destroy(remove?: Boolean): void;
}
